# This time Ctrl and Stx5 are pooled

library(tidyverse)
library(ggpubr)
library(scales)

PL0501 <- read.csv("../../processed/fibroblasts/transferases/190603_PL050.1_coloc_tidy.csv", sep = ";")
PL0502 <- read.csv("../../processed/fibroblasts/transferases/190603_PL050.2_coloc_tidy.csv", sep = ";")
PL0503 <- read.csv("../../processed/fibroblasts/transferases/190806_PL050.3_coloc_tidy.csv", sep = ";")

df <- rbind(PL0501, PL0502)
df <- rbind(df, PL0503)
df$Line <- factor(df$Line, levels = c("14197", "14242", "14230", "14235"))
df$Condition <- factor(df$Condition, levels = c("Ctrl", "Stx5"))
df$Marker <- factor(df$Marker, levels = c("gm130", "TGN46", "ZFPL1"))
df$Enzyme <- factor(df$Enzyme, levels = c("GALNT2", "MAN2A1", "MGAT1", "ST6GAL1"))

levels(df$Condition)[levels(df$Condition) == "Stx5"] <- "Stx5M55V"

intmag95 <- function(x) {
    sem <- sd(x) / sqrt(length(x))
    t <- qt(0.975, df = length(x) - 1)
    return(sem * t)
}

summary_ci <- function(x) {
    m <- mean(x)
    ymin <- m - intmag95(x)
    ymax <- m + intmag95(x)
    return(c(y = m, ymin = ymin, ymax = ymax))

}

generate_transferase_plot <- function(data, enzyme, marker) {
    stats_results <- data %>%
        filter(Enzyme == enzyme) %>%
        filter(Marker == marker) %>%
        compare_means(value ~ Condition, data = ., method = "t.test", p.adjust.method = "bonferroni")
    
    stats_compare_df <- stats_results %>%
        filter(p.signif != "ns") %>%
        select(group1, group2) %>%
        rowwise() %>%
        do(comparisons = c(.$group1, .$group2))

    p <- data %>%
        filter(Enzyme == enzyme) %>%
        filter(Marker == marker) %>%
        ggplot(., aes(x = Condition, y = value)) +
        geom_violin(trim = TRUE, aes(fill = Condition, color = Condition)) +
        stat_summary(fun.data = summary_ci, geom = "pointrange", shape = 20, size = 0.3, colour = "gray18", fatten = 0.2, position=position_dodge(1), show.legend=FALSE) +
        ylab(sprintf("Pearson Correlation Coefficient\n %s %s", enzyme, marker)) +
        scale_color_brewer(palette = "Dark2") +
        scale_fill_brewer(palette = "Dark2") +
        scale_y_continuous( breaks=c(0.0, 0.2, 0.4, 0.6, 0.8, 1.0), limits=c(-0.2, 1.1)) +
        theme(legend.position = "none")


    
    ggsave(sprintf("%s_%s_pooled.pdf", enzyme, marker), p, device = cairo_pdf, dpi = 600, width = 26.5, height = 56.00, units = "mm")


}

for (enzyme in levels(df$Enzyme)) {
    subset <- df %>%
        filter(Enzyme == enzyme) %>%
        droplevels()
    for (marker in levels(subset$Marker)) {
        generate_transferase_plot(df, enzyme, marker)
    }
}

ttest_chars <- capture.output(for (enzyme in levels(df$Enzyme)) {
    subset <- df %>%
        filter(Enzyme == enzyme) %>%
        droplevels()
    for (marker in levels(subset$Marker)) {
        ttest <- df %>%
            filter(Enzyme == enzyme) %>%
            filter(Marker == marker) %>%
            t.test(value ~ Condition, data = ., var.equal = TRUE)
        print(enzyme)
        print(marker)
        print(ttest)
    }       
})


writeLines(ttest_chars, con = file("Transferase_stats.txt"))

chars <- capture.output(for (enzyme in levels(df$Enzyme)) {
    subset <- df %>%
        filter(Enzyme == enzyme) %>%
        droplevels()
    for (marker in levels(subset$Marker)) {
        print(enzyme)
        print(marker)
        print(df %>%
            filter(Enzyme == enzyme) %>%
            filter(Marker == marker) %>%
            group_by(Condition) %>%
            summarise(N = length(value),
                     Median = median(value),
                     Mean = mean(value),
                     SD = sd(value),
                     Var = var(value)))
    }
})

writeLines(chars, con = file("Transferase_vals.txt"))
