library(tidyverse)
library(ggpubr)
library(RColorBrewer)


# Helper functions
intmag95 <- function(x) {
    sem <- sd(x) / sqrt(length(x))
    t <- qt(0.975, df = length(x) - 1)
    return(sem * t)
}

summary_ci <- function(x) {
    m <- mean(x)
    ymin <- m - intmag95(x)
    ymax <- m + intmag95(x)
    return(c(y = m, ymin = ymin, ymax = ymax))
}


m567 <- read.csv("../../processed/HeLa/H89/190925_M567_tidy_DMSO.csv", sep = ";")
m596 <- read.csv("../../processed/HeLa/H89/190925_M596_tidy_DMSO.csv", sep = ";")
m567$Line <- factor(m567$Line, levels = c("HeLa", "B1A7", "C1F4"))
m596$Line <- factor(m596$Line, levels = c("HeLa", "B1A7", "C1F4"))
m567$Label <- factor(m567$Label, levels = c("HeLa_DMSO", "HeLa_H89", "HeLa_Washout","B1A7_DMSO", "B1A7_H89", "B1A7_Washout","C1F4_DMSO", "C1F4_H89", "C1F4_Washout"))
m596$Label <- factor(m596$Label, levels = c("HeLa_DMSO", "HeLa_H89", "HeLa_Washout","B1A7_DMSO", "B1A7_H89", "B1A7_Washout","C1F4_DMSO", "C1F4_H89", "C1F4_Washout"))
df <- rbind(m567, m596)
df <- df %>%
    mutate(
        Condition = case_when(
            grepl(pattern = "HeLa", Line) ~ "WT",
            grepl(pattern = "C1F4|B1A7", Line) ~ "Stx5ΔL"))
df$Condition <- factor(df$Condition, levels = c("WT", "Stx5ΔL"))

deltaL_palette <- brewer.pal(8, "Dark2")[c(1,3)]

p <- ggplot(df, aes(x = Condition, y = Value, fill = Condition, color = Condition)) +
        geom_violin(trim = TRUE, size=0.2) +
        stat_summary(fun.data = summary_ci, geom = "pointrange", shape = 20, size = 0.3, colour = "gray18", position=position_dodge(0.9), fatten = 0.2, show.legend=FALSE) +
        #stat_compare_means(comparisons = comparisons, method = "t.test", label = "p.signif", tip.length = 0.01, step.increase = 0.05, bracket.size = 0.05, hide.ns = TRUE, size = 1, label.size = 0.02, label.padding=unit(0.1, "mm")) +
        labs(y = "Relative fluorescent intensity (AU)") +
        scale_color_manual(values = deltaL_palette) +
        scale_fill_manual(values = deltaL_palette) +
        facet_grid(~Treatment) +
        theme(axis.text.x = element_text(angle = 90, hjust = 0.95, vjust = 0.5))

ggsave("H89_DMSO.pdf", p, device = cairo_pdf, dpi = 600, width = 55.5, height = 56.00, units = "mm") 

vals <- df %>%
    filter(Value != 0) %>%
    group_by(Condition, Treatment) %>%
        summarise(N = length(Value),
            Median = median(Value),
            Mean = mean(Value),
            SD = sd(Value),
            Var = var(Value))

writeLines(capture.output(print(vals)), "H89_vals.txt")

stats <- capture.output(for (treatment in levels(df$Treatment)) {
    h89wilcox <- df %>% 
        filter(Treatment == treatment) %>%
        wilcox.test(Value ~ Condition, data = .)

    print(treatment)
    print(h89wilcox)
}
)

writeLines(stats, "H89_stats.txt")