library(tidyverse)
library(broom)
library(openxlsx)
library(RColorBrewer)


intmag95 <- function(x) {
    sem <- sd(x) / sqrt(length(x))
    t <- qt(0.975, df = length(x) - 1)
    return(sem * t)
}

summary_ci <- function(x) {
    m <- mean(x)
    ymin <- m - intmag95(x)
    ymax <- m + intmag95(x)
    return(c(y = m, ymin = ymin, ymax = ymax))
}

df_m532 <- read.xlsx("../../processed/HeLa/VSVG/190924_M532_wide.xlsx")
df_m532_tidy <- df_m532 %>%
                gather("CellIdent", "Value", -Time, factor_key = TRUE)

df_m535 <- read.xlsx("../../processed/HeLa/VSVG/190924_M535_wide.xlsx")
df_m535_tidy <- df_m535 %>%
                gather("CellIdent", "Value", -Time, factor_key = TRUE)

df <- rbind(df_m532_tidy, df_m535_tidy)

df <- df %>%
    mutate(
        Line = case_when(
           grepl(pattern = "HeLa|Hela", CellIdent) ~ "HeLa",
            grepl(pattern = "C1F4", CellIdent) ~ "C1F4",
            grepl(pattern = "B1A7", CellIdent) ~ "B1A7"))

df <- df %>%
    mutate(
        Condition = case_when(
            grepl(pattern = "HeLa", Line) ~ "WT",
            grepl(pattern = "C1F4|B1A7", Line) ~ "Stx5ΔL"))

df$Line <- factor(df$Line, levels = c("HeLa", "B1A7", "C1F4"))
df$Condition <- factor(df$Condition, levels = c("WT", "Stx5ΔL"))

df_mean <- df %>%
            group_by(Condition, Time) %>%
            summarise(mean = mean(Value), max = (mean(Value) + intmag95(Value)), min = (mean(Value) - intmag95(Value)))
# deltaL colours
deltaL_palette <- brewer.pal(8, "Dark2")[c(1,3)]

p <- df_mean %>%
     filter(Time <= 20) %>%
    ggplot(., aes(x = Time, y = mean, fill = Condition, color = Condition)) +
    geom_ribbon(aes(ymin = min, ymax = max), alpha = 0.3, colour = NA) +
    geom_line() +
    # scale_y_continuous(breaks = c(0.2, 0.4, 0.6, 0.8, 1)) +
    labs(y = "Relative fluorescent intensity (AU)", x = "Time (min)") +
    scale_color_manual(values = deltaL_palette) +
    scale_fill_manual(values = deltaL_palette) +
    theme(axis.text.x = element_text(angle = 0, hjust = 0.5), axis.title.x = element_text())
ggsave("Mean_SEM.pdf", p, device = cairo_pdf, dpi = 600, width = 42.5, height = 56.00, units = "mm")

WT_fits <- df %>%
    filter(Condition == "WT", Time >= 3.0 & Time <= 20.0) %>%
    group_by(CellIdent) %>%
    do(fit = lm(Value ~ Time, data = .))
deltaL_fits <- df %>%
    filter(Condition == "Stx5ΔL", Time >= 3.0 & Time <= 20.0) %>%
    group_by(CellIdent) %>%
    do(fit = lm(Value ~ Time, data = .))

WT_coef <- tidy(WT_fits, fit) %>%
                filter(term == "Time") %>%
                select(estimate)
deltaL_coef <- tidy(deltaL_fits, fit) %>%
                filter(term == "Time") %>%
                select(estimate)

WT_coef$Condition = rep("WT", length(WT_coef$estimate))
deltaL_coef$Condition = rep("Stx5ΔL", length(deltaL_coef$estimate))
coef_df <- rbind(WT_coef, deltaL_coef)
coef_df$log2Estimate <- log2(coef_df$estimate)
coef_df$Condition <- factor(coef_df$Condition, levels = c("WT", "Stx5ΔL"))
vals_chars <- capture.output(coef_df %>%
                                ungroup() %>%
                                drop_na() %>%
                                group_by(Condition) %>%
                                summarise(N = length(estimate),
                                Median = median(estimate),
                                Mean = mean(estimate),
                                SD = sd(estimate),
                                Var = var(estimate),
                                log2Median = median(log2Estimate),
                                log2Mean = mean(log2Estimate),
                                log2SD = sd(log2Estimate),
                                log2Var = var(log2Estimate)))


writeLines(vals_chars, "VSVG_vals.txt")

ttest <- t.test(log2Estimate ~ Condition, data = coef_df, var.equal = TRUE)
ttest_chars <- capture.output(print(ttest))
writeLines(ttest_chars, "VSVG_stats.txt")

p <- ggplot(coef_df, aes(x = Condition, y = log2Estimate)) +
    geom_violin(trim = TRUE, aes(fill = Condition, color = Condition)) +
    stat_summary(fun.data = summary_ci, geom = "pointrange", shape = 20, size = 0.3, colour = "gray18", fatten = 0.2, position=position_dodge(1), show.legend=FALSE) +
    labs(y = "log2(VSVG-EGFP trafficking rate)") +
    scale_color_manual(values = deltaL_palette) +
    scale_fill_manual(values = deltaL_palette) +
    theme(axis.text.x = element_text(angle = 90, hjust = 0.95, vjust = 0.5), legend.position = "none")
ggsave("Slope.pdf", p, device = cairo_pdf, dpi = 600, width = 42.5, height = 56.00, units = "mm") 
