
library(tidyverse)
library(ggpubr)
library(scales)

source("../../../../../R_themes//myTheme.R")

df <- read.csv("raw_data/RinseTEMData.csv", sep = ";", stringsAsFactors = TRUE)

df$Line <- factor(df$Line, levels = c("14197", "14242", "14230", "14235"))
df$Condition <- factor(df$Condition, levels = c("Ctrl", "Stx5M55V"))
df$Organelle <- factor(df$Organelle, levels = c("ER", "Golgi"))
df$Readout <- factor(df$Readout, levels = c("Area", "Ratio"))

intmag95 <- function(x) {
    sem <- sd(x) / sqrt(length(x))
    t <- qt(0.975, df = length(x) - 1)
    return(sem * t)
}

summary_ci <- function(x) {
    m <- mean(x)
    ymin <- m - intmag95(x)
    ymax <- m + intmag95(x)
    return(c(y = m, ymin = ymin, ymax = ymax))

}

generate_tem_plot <- function(data, organelle, readout) {
    stats_results <- data %>%
        filter(Organelle == organelle) %>%
        filter(Readout == readout) %>%
        compare_means(Value ~ Condition, data = ., method = "t.test", p.adjust.method = "bonferroni")
    
    stats_compare_df <- stats_results %>%
        filter(p.signif != "ns") %>%
        select(group1, group2) %>%
        rowwise() %>%
        do(comparisons = c(.$group1, .$group2))

    p <- data %>%
        filter(Organelle == organelle) %>%
        filter(Readout == readout) %>%
        ggplot(., aes(x = Condition, y = Value)) +
        geom_violin(trim = TRUE, aes(fill = Condition, color = Condition)) +
        stat_summary(fun.data = summary_ci, geom = "pointrange", shape = 20, size = 0.3, colour = "gray18", fatten = 0.2, position=position_dodge(1), show.legend=FALSE) +
        # ylab(sprintf("Pearson Correlation Coefficient\n %s %s", enzyme, marker)) +
        scale_color_brewer(palette = "Dark2") +
        scale_fill_brewer(palette = "Dark2") +
        # scale_y_continuous( breaks=c(0.0, 0.2, 0.4, 0.6, 0.8, 1.0), limits=c(-0.2, 1.1)) +
        mytheme +
        theme(legend.position = "none")


    
    ggsave(sprintf("%s_%s.pdf", organelle, readout), p, device = cairo_pdf, dpi = 600, width = 26.5, height = 56.00, units = "mm")
}

for (organelle in levels(df$Organelle)) {
    subset <- df %>%
        filter(Organelle == organelle) %>%
        droplevels()
    for (readout in levels(subset$Readout)) {
        generate_tem_plot(df, organelle, readout)
    }
}

ttest_chars <- capture.output(for (organelle in levels(df$Organelle)) {
    subset <- df %>%
        filter(Organelle == organelle) %>%
        droplevels()
    for (readout in levels(subset$Readout)) {
        ttest <- df %>%
            filter(Organelle == organelle) %>%
            filter(Readout == readout) %>%
            t.test(Value ~ Condition, data = ., var.equal = TRUE)
        print(organelle)
        print(readout)
        print(ttest)
    }       
})


writeLines(ttest_chars, con = file("TEM_stats.txt"))

chars <- capture.output(for (organelle in levels(df$Organelle)) {
    subset <- df %>%
        filter(Organelle == organelle) %>%
        droplevels()
    for (readout in levels(subset$Readout)) {
        print(organelle)
        print(readout)
        print(df %>%
            filter(Organelle == organelle) %>%
            filter(Readout == readout) %>%
            group_by(Condition) %>%
            summarise(N = length(Value),
                     Median = median(Value),
                     Mean = mean(Value),
                     SD = sd(Value),
                     Var = var(Value)))
    }
})

writeLines(chars, con = file("TEM_vals.txt"))