library(tidyverse)
library(rstatix)

source("../../../../../R_themes//myTheme.R")

washout1 <- read.csv("raw_data/201014_Washout1.csv", sep = ',', stringsAsFactors = TRUE)
washout2 <- read.csv("raw_data/201015_Washout2.csv", sep = ',', stringsAsFactors = TRUE)
df <- rbind(washout1, washout2)

df$Line <- factor(df$Line, levels = c("14197", "14242", "14230", "14235"))
df$Condition <- factor(df$Condition, levels = c("Ctrl", "Stx5M55V"))
df$Timepoint <- factor(df$Timepoint, levels = c("DMSO", "0mins", "30mins", "60mins", "90mins"))

intmag95 <- function(x) {
    sem <- sd(x) / sqrt(length(x))
    t <- qt(0.975, df = length(x) - 1)
    return(sem * t)
}

summary_ci <- function(x) {
    m <- mean(x)
    ymin <- m - intmag95(x)
    ymax <- m + intmag95(x)
    return(c(y = m, ymin = ymin, ymax = ymax))
}

plot_df <- df %>%
    filter(Timepoint != "DMSO") %>%
    mutate(Time = case_when(
        Timepoint == "0mins" ~ 0,
        Timepoint == "30mins" ~ 30,
        Timepoint == "60mins" ~ 60,
        Timepoint == "90mins" ~ 90))

p <- ggplot(plot_df, aes(x = Time, y = Value, fill = Condition, color = Condition)) +
        geom_jitter(width = 5, alpha = 0.3, stroke = 0, size = 0.7) +
        stat_summary(aes(group = interaction(Time, Condition), color = Condition), fun.data = summary_ci, geom = "pointrange", shape = 20, size = 0.3, fatten = 0.2, show.legend=FALSE, position= position_dodge(width = 0.5)) +
        geom_smooth(data = plot_df, method = "loess", span = 0.8, alpha = 0.3, linetype = 0) +
        # stat_smooth(data = plot_df, method = "loess", span = 0.8, alpha = 0.3, size = 1, se = FALSE) +
        geom_line(stat = "smooth", method = "loess", alpha = 0.5, size = 1, span = 0.8) +
        ylab(expression("Relative GALNT2 log"[10]*"(intensity)")) +
        xlab("Time after washout (mins)") +
        scale_x_continuous(breaks=c(0, 30, 60, 90)) +
        scale_color_brewer(palette = "Dark2") +
        scale_fill_brewer(palette = "Dark2") +
       # scale_y_continuous(limits=c(-2.5, 1.0)) +
        mytheme +
        theme(line=element_line(size=0.5),
                text=element_text(size=8),
                plot.title = element_text(hjust = 0.5),
                #legend.justification = c("right", "top"),
                #legend.box.just = "right",
                legend.position = "none",
                axis.text=element_text(size=6), 
                axis.text.x = element_text(angle = 0, hjust = 0.5, vjust = 0.5), 
                axis.title.x = element_text(),
                axis.title.y = element_text(margin = unit(c(0.01,0.01,0.01,0.01),"cm")))

ggsave("BFA_timecourse.pdf", p, device = cairo_pdf, dpi = 600, width = 70, height = 56, units = "mm")

generate_timepoint_plot <- function(data, timepoint) {
     p1 <- data %>%
        filter(Timepoint == timepoint) %>%
        ggplot(., aes(x = Condition, y = Value)) +
        geom_violin(trim = TRUE, aes(fill = Condition, color = Condition)) +
        stat_summary(fun.data = summary_ci, geom = "pointrange", shape = 20, size = 0.3, colour = "gray18", fatten = 0.2, position=position_dodge(1), show.legend=FALSE) +
        ylab(expression("Relative GALNT2 log"[10]*"(intensity)")) +
        scale_color_brewer(palette = "Dark2") +
        scale_fill_brewer(palette = "Dark2") +
        scale_y_continuous(limits=c(0.5, 1.0)) +
        mytheme +
        theme(legend.position = "none")

    ggsave(sprintf("BFA_%s.pdf", timepoint), p1, device = cairo_pdf, dpi = 600, width = 26.5, height = 56.00, units = "mm")
}

generate_timepoint_plot(df, "0mins")
generate_timepoint_plot(df, "30mins")
generate_timepoint_plot(df, "60mins")
generate_timepoint_plot(df, "90mins")

stats_df <- plot_df %>%
    group_by(Time) %>%
    t_test(Value ~ Condition) %>%
    adjust_pvalue(method = "bonferroni") %>%
    add_significance("p.adj")

write.csv(stats_df, "BFA_washout_stats.csv")