library(tidyverse)
library(broom)
library(rstatix)

source("../../../../R_themes/myTheme.R")

df <- read.csv("Data/210820_kinetics.csv", stringsAsFactors = TRUE)
df <- as_tibble(df)

# Helper functions
normalize <- function(x){
  return((x-min(x)) / (max(x)-min(x)))
}

intmag95 <- function(x) {
    sem <- sd(x) / sqrt(length(x))
    t <- qt(0.975, df = length(x) - 1)
    return(sem * t)
}

summary_ci <- function(x) {
    m <- mean(x)
    ymin <- m - intmag95(x)
    ymax <- m + intmag95(x)
    return(c(y = m, ymin = ymin, ymax = ymax))
}

levels(df$Isoform) <- list("Stx5S"="S", "Stx5L"="L", "Stx5LΔER"="D")

summary_clean <- df %>%
        group_by(cellIdent) %>%
        ungroup() %>%
        group_by(Isoform, Time) %>%
        summarise(mean = mean(normToFirstFrame), max = (mean(normToFirstFrame) + intmag95(normToFirstFrame)), min = (mean(normToFirstFrame) - intmag95(normToFirstFrame)))

raw_clean <- df %>%
        group_by(cellIdent) %>%
        ungroup()

S_lm <- summary_clean %>%
    filter(Isoform == "Stx5S", Time > 20.0) %>%
    lm(mean ~ Time, data = .)
S_new <- data.frame(Time = seq(20.0, 60, 0.1))
S_predict <- predict(S_lm, S_new, se.fit = FALSE)
S_new$predict <- S_predict
S_new$Isoform <- rep("Stx5S", length(S_new$predict))

S_lm_er <- summary_clean %>%
  filter(Isoform == "Stx5S", Time < 20.0) %>%
  lm(mean ~ Time, data = .)
S_new_er <- data.frame(Time = seq(0, 20.0, 0.1))
S_predict_er <- predict(S_lm_er, S_new_er, se.fit = FALSE)
S_new_er$predict <- S_predict_er
S_new_er$Isoform <- rep("Stx5S", length(S_new_er$predict))


L_lm <- summary_clean %>%
    filter(Isoform == "Stx5L", Time > 20.0) %>%
    lm(mean ~ Time, data = .)
L_new <- data.frame(Time = seq(20.0, 60, 0.1))
L_predict <- predict(L_lm, L_new, se.fit = FALSE)
L_new$predict <- L_predict
L_new$Isoform <- rep("Stx5L", length(L_new$predict))

L_lm_er <- summary_clean %>%
  filter(Isoform == "Stx5L", Time < 20.0) %>%
  lm(mean ~ Time, data = .)
L_new_er <- data.frame(Time = seq(0, 20, 0.1))
L_predict_er <- predict(L_lm_er, L_new_er, se.fit = FALSE)
L_new_er$predict <- L_predict_er
L_new_er$Isoform <- rep("Stx5L", length(L_new_er$predict))

D_lm <- summary_clean %>%
    filter(Isoform == "Stx5LΔER", Time > 20.0) %>%
    lm(mean ~ Time, data = .)
D_new <- data.frame(Time = seq(20.0, 60, 0.1))
D_predict <- predict(D_lm, D_new, se.fit = FALSE)
D_new$predict <- D_predict
D_new$Isoform <- rep("Stx5LΔER", length(D_new$predict))

D_lm_er <- summary_clean %>%
  filter(Isoform == "Stx5LΔER", Time < 20.0) %>%
  lm(mean ~ Time, data = .)
D_new_er <- data.frame(Time = seq(0, 20, 0.1))
D_predict_er <- predict(D_lm_er, D_new_er, se.fit = FALSE)
D_new_er$predict <- D_predict_er
D_new_er$Isoform <- rep("Stx5LΔER", length(D_new_er$predict))

lm_df <- rbind(S_new, L_new)
lm_df <- rbind(lm_df, D_new)
lm_df$Isoform <- as.factor(lm_df$Isoform)
levels(lm_df$Isoform) <- list("Stx5S"="Stx5S", "Stx5L"="Stx5L", "Stx5LΔER"="Stx5LΔER")

lm_df_er <- rbind(S_new_er, L_new_er)
lm_df_er <- rbind(lm_df_er, D_new_er)
lm_df_er$Isoform <- as.factor(lm_df_er$Isoform)
levels(lm_df_er$Isoform) <- list("Stx5S"="Stx5S", "Stx5L"="Stx5L", "Stx5LΔER"="Stx5LΔER")

# Mean + 95% CI
p <- ggplot(summary_clean, aes(x = Time, y = mean, fill = Isoform, color = Isoform)) +
    geom_ribbon(aes(ymin = min, ymax = max), alpha = 0.2, colour = NA) +
    # geom_line(linetype = 2, alpha = 0.5) +
    # geom_smooth(method = "loess", se = FALSE, span = 0.1, linetype = "dotted", aes(alpha = 0.2)) +
    geom_line(stat="smooth", method = "loess", span = 0.1, size = 0.5, linetype = "dashed", alpha = 0.5) +
    geom_line(data = lm_df, aes(x = Time, y = predict)) +
    geom_line(data = lm_df_er, aes(x = Time, y = predict)) +
    # scale_y_continuous(breaks = c(0.2, 0.4, 0.6, 0.8, 1)) +
    labs(y = "Relative Stx5 intensity at Golgi (AU)", x = "Time (min)") +
    scale_color_brewer(palette = "Dark2") +
    scale_fill_brewer(palette = "Dark2") +
    mytheme +
    theme(axis.text.x = element_text(angle = 0, hjust = 0.5), axis.title.x = element_text())
ggsave("Mean_SEM.pdf", p, device = cairo_pdf, dpi = 600, width = 42.5, height = 56.00, units = "mm")



S_fits <- raw_clean %>%
    filter(Isoform == "Stx5S", Time > 20.0) %>%
    nest(data = -cellIdent) %>%
    mutate(
        fit = map(data, ~ lm(normToFirstFrame ~ Time, data = .x)),
        tidied = map(fit, tidy)) %>%
    unnest(tidied)
L_fits <- raw_clean %>%
    filter(Isoform == "Stx5L", Time > 20.0) %>%
    nest(data = -cellIdent) %>%
    mutate(
        fit = map(data, ~ lm(normToFirstFrame ~ Time, data = .x)),
        tidied = map(fit, tidy)) %>%
    unnest(tidied)
D_fits <- raw_clean %>%
    filter(Isoform == "Stx5LΔER", Time > 20.0) %>%
    nest(data = -cellIdent) %>%
    mutate(
        fit = map(data, ~ lm(normToFirstFrame ~ Time, data = .x)),
        tidied = map(fit, tidy)) %>%
    unnest(tidied)

S_fits_ER <- raw_clean %>%
  filter(Isoform == "Stx5S", Time < 20.0) %>%
  nest(data = -cellIdent) %>%
  mutate(
    fit = map(data, ~ lm(normToFirstFrame ~ Time, data = .x)),
    tidied = map(fit, tidy)) %>%
  unnest(tidied)
L_fits_ER <- raw_clean %>%
  filter(Isoform == "Stx5L", Time < 20.0) %>%
  nest(data = -cellIdent) %>%
  mutate(
    fit = map(data, ~ lm(normToFirstFrame ~ Time, data = .x)),
    tidied = map(fit, tidy)) %>%
  unnest(tidied)
D_fits_ER <- raw_clean %>%
  filter(Isoform == "Stx5LΔER", Time < 20.0) %>%
  nest(data = -cellIdent) %>%
  mutate(
    fit = map(data, ~ lm(normToFirstFrame ~ Time, data = .x)),
    tidied = map(fit, tidy)) %>%
  unnest(tidied)

S_coef <- S_fits %>%
filter(term == "Time") %>%
select(estimate)

L_coef <- L_fits %>%
filter(term == "Time") %>%
select(estimate)

D_coef <- D_fits %>%
filter(term == "Time") %>%
select(estimate)

S_coef_ER <- S_fits_ER %>%
  filter(term == "Time") %>%
  select(estimate)

L_coef_ER <- L_fits_ER %>%
  filter(term == "Time") %>%
  select(estimate)

D_coef_ER <- D_fits_ER %>%
  filter(term == "Time") %>%
  select(estimate)

S_coef$Isoform <- rep("Stx5S", length(S_coef$estimate))
L_coef$Isoform <- rep("Stx5L", length(L_coef$estimate))
D_coef$Isoform <- rep("Stx5LΔER", length(D_coef$estimate))
coef_df <- rbind(S_coef, L_coef)
coef_df <- rbind(coef_df, D_coef)

S_coef_ER$Isoform <- rep("Stx5S", length(S_coef_ER$estimate))
L_coef_ER$Isoform <- rep("Stx5L", length(L_coef_ER$estimate))
D_coef_ER$Isoform <- rep("Stx5LΔER", length(D_coef_ER$estimate))
coef_df_ER <- rbind(S_coef_ER, L_coef_ER)
coef_df_ER <- rbind(coef_df_ER, D_coef_ER)

coef_df$Isoform <- factor(coef_df$Isoform, levels = c("Stx5S", "Stx5L", "Stx5LΔER"))
coef_df_ER$Isoform <- factor(coef_df_ER$Isoform, levels = c("Stx5S", "Stx5L", "Stx5LΔER"))

vals <- coef_df %>%
    group_by(Isoform) %>%
    summarise(N = length(estimate),
                Median = median(estimate),
                Mean = mean(estimate),
                SD = sd(estimate),
                Var = var(estimate))
writeLines(capture.output(print(vals)), "Kinetics_vals.txt")

p <- ggplot(coef_df, aes(x = Isoform, y = estimate)) +
    geom_violin(trim = TRUE, aes(fill = Isoform, color = Isoform)) +
    stat_summary(fun.data = summary_ci, geom = "pointrange", shape = 20, size = 0.3, colour = "gray18", fatten = 0.2, position=position_dodge(1), show.legend=FALSE) +
    labs(y = "Rate of Stx5 Golgi exit") +
    scale_color_brewer(palette = "Dark2") +
    scale_fill_brewer(palette = "Dark2") +
    scale_y_continuous(limits = c(-0.05, 0.2), breaks = c(-0.1, 0, 0.1, 0.2)) +
    mytheme +
    theme(axis.text.x = element_text(angle = 90, hjust = 0.95, vjust = 0.5), legend.position = "none")
ggsave("Slope_Golgi.pdf", p, device = cairo_pdf, dpi = 600, width = 42.5, height = 56.00, units = "mm") 

p <- ggplot(coef_df_ER, aes(x = Isoform, y = estimate)) +
  geom_violin(trim = TRUE, aes(fill = Isoform, color = Isoform)) +
  stat_summary(fun.data = summary_ci, geom = "pointrange", shape = 20, size = 0.3, colour = "gray18", fatten = 0.2, position=position_dodge(1), show.legend=FALSE) +
  labs(y = "Rate of Stx5 ER exit") +
  scale_color_brewer(palette = "Dark2") +
  scale_fill_brewer(palette = "Dark2") +
  scale_y_continuous(limits = c(-0.05, 0.2), breaks = c(-0.1, 0, 0.1, 0.2)) +
  mytheme +
  theme(axis.text.x = element_text(angle = 90, hjust = 0.95, vjust = 0.5), legend.position = "none")

ggsave("Slope_ER.pdf", p, device = cairo_pdf, dpi = 600, width = 42.5, height = 56.00, units = "mm") 


kineticsaov <- aov(estimate ~ Isoform, data = coef_df)
eraov <- aov(estimate ~ Isoform, data = coef_df_ER)

stats_chars <- capture.output(print(summary(kineticsaov)), print(TukeyHSD(kineticsaov)))
writeLines(stats_chars, "Kinetics_stats.txt")

df %>%
  filter(Time == 20) %>%
  dunn_test(normToFirstFrame ~ Isoform)

TukeyHSD(kineticsaov)
TukeyHSD(eraov)
